#!/usr/bin/env python
# -*- mode: python; coding: utf-8 -*-
#
# Copyright 2011 Fumihiro Kato <fumi@fumi.me>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from google.appengine.api import urlfetch
from google.appengine.ext import db
from google.appengine.ext import webapp
from google.appengine.ext.webapp import util

from cgi import parse_qs
from django.utils import simplejson
import os
import sys
import time

import oauth2 as oauth
from appengine_utilities import sessions
from thrift.protocol import TBinaryProtocol
from thrift.transport import THttpClient
from evernote.edam.notestore import NoteStore
from evernote.edam.notestore.NoteStore import NotesMetadataResultSpec
from evernote.edam.notestore.ttypes import NoteFilter

## 取得したウェブアプリケーション用キーで置き換える
OAUTH_CONSUMER_KEY = "yourkey"
OAUTH_CONSUMER_SECRET = "yoursecret"

#EVERNOTE_BASE_URI = "https://www.evernote.com"
EVERNOTE_BASE_URI = "https://sandbox.evernote.com"
TOKEN_URI = EVERNOTE_BASE_URI + "/oauth"
AUTHORIZATION_URI = EVERNOTE_BASE_URI + "/OAuth.action"
NOTESTORE_BASE_URI = EVERNOTE_BASE_URI + "/edam/note/"

## GAEにアップロードするときは対応するサーバ名に変更する
## ローカルテスト時にはポート番号を適切に設定する
APP_BASE_URI = "http://localhost:8081"
TOKEN_CALLBACK_URI = APP_BASE_URI + "/oauth/token_callback"

def check_authenticated_user(session):
    """OAuth済でセッション内に必要なデータがあることを確認
    """
    for key in ["oauth_token", "edam_shard", "edam_userId"]:
        if key not in session or session[key] == "":
            sys.stderr.write('OAuth認証を先に行う必要があります．\n')
            return False
    return True

def get_notestore(session):
    """セッション内の値からNotestoreを取得
    """
    notestore_uri =  NOTESTORE_BASE_URI + session["edam_shard"]
    notestore_http_client = THttpClient.THttpClient(notestore_uri)
    notestore_protocol = TBinaryProtocol.TBinaryProtocol(notestore_http_client)
    return NoteStore.Client(notestore_protocol)

class IndexHandler(webapp.RequestHandler):
    """トップページ表示
    """
    def get(self):
        session = sessions.Session()
        self.response.headers["Content-Type"] = "text/html; charset=utf-8"
        self.response.out.write('<html><head><title>EDAM OAuth デモ</title></body>')
        self.response.out.write('<body><h1>EDAM OAuth デモ</h1><ul>')
        self.response.out.write('<li><a href="/oauth/token">トークン取得</a></li>')
        if "oauth_token" in session:
            self.response.out.write('<li><a href="/list">ノートブック一覧</a></li>')
            self.response.out.write('<li><a href="/timeline">タイムライン表示</a></li>')
        self.response.out.write('<li><a href="/reset">リセット</a></li>')
        self.response.out.write('</ul></body></html>')

class OAuthTokenHandler(webapp.RequestHandler):
    """OAuthの認証手続き"""
    def get(self):
        ## ウェブアプリケーション用キーの設定
        consumer = oauth.Consumer(key=OAUTH_CONSUMER_KEY, secret=OAUTH_CONSUMER_SECRET)
        ## トークン取得後にOAuthサーバから転送してもらうURIの設定
        params = { "oauth_callback": TOKEN_CALLBACK_URI }
        ## 最初の要求作成: この段階ではトークンはなし
        request = oauth.Request.from_consumer_and_token(
            consumer, token=None, http_method="GET", http_url=TOKEN_URI, parameters=params)
        ## シグニチャの作成
        request.sign_request(oauth.SignatureMethod_HMAC_SHA1(), consumer, None)
        ## 最初の要求
        uri = request.to_url()
        result = urlfetch.fetch(uri)
        if result.status_code != 200:
            self.redirect("/error")
            return

        ## レスポンスから仮証明(リクエストトークン)を抽出
        token = oauth.Token.from_string(result.content)
        ## OAuthサーバのユーザの認証・同意画面に飛ばす
        request = oauth.Request.from_token_and_callback(
            token=token, callback=TOKEN_CALLBACK_URI, http_url=AUTHORIZATION_URI)
        uri = request.to_url()
        self.redirect(uri)

class OAuthTokenCallbackHandler(webapp.RequestHandler):
    """OAuthから戻ってきた後の処理"""
    def get(self):
        session = sessions.Session()
	## ユーザが承認した場合はoauth_tokenとoauth_verifierが返ってくる
        oauth_token = self.request.get("oauth_token")
        oauth_verifier = self.request.get("oauth_verifier")
        ## シークレットは使用しないので空で良い
        token = oauth.Token(oauth_token, "")
        ## oauth_verifierは必須
        token.set_verifier(oauth_verifier)

        ## トークン証明(アクセストークン)の取得
        consumer = oauth.Consumer(key=OAUTH_CONSUMER_KEY, secret=OAUTH_CONSUMER_SECRET)
        client = oauth.Client(consumer, token)
        ## oauth.SignatureMethod_HMAC_SHA1()では動かない (要確認?)
        client.set_signature_method(oauth.SignatureMethod_PLAINTEXT())
        response, content = client.request(TOKEN_URI)
        if response.status != 200:
            self.redirect("/error")
            return

        ## トークン証明(アクセストークン)などの抽出
        ## NoteStoreにアクセスするのに必要なのでセッションとして保持
        params = parse_qs(content)
        session["oauth_token"] = params["oauth_token"][0]
        session["edam_shard"] = params["edam_shard"][0]
        session["edam_userId"] = params["edam_userId"][0]
        self.redirect("/")

class ListHandler(webapp.RequestHandler):
    """ノートブックの一覧を表示"""
    def get(self):
        session = sessions.Session()
        if not check_authenticated_user(session):
            self.redirect("/error")
            return

        ## NoteStoreからnotebooksを取得
        notestore = get_notestore(session)
        notebooks = notestore.listNotebooks(session["oauth_token"])
        self.response.out.write('<html><head><title>ノートブック一覧</title></head>')
        self.response.out.write('<body><h1>ノートブック一覧</h1>')
        self.response.out.write('<p>%d個のノートブック</p><ul>' % len(notebooks))
        for notebook in notebooks:
            self.response.out.write('<li>%s</li>' % notebook.name)
        self.response.out.write('</ul><p><a href="/">トップページに戻る</a></p></body></html>')

class TimelineHandler(webapp.RequestHandler):
    """ノートをタイムライン上に表示"""
    def get(self):
        session = sessions.Session()
        if not check_authenticated_user(session):
            self.redirect("/error")
            return

        self.response.out.write('<html><head>')
        self.response.out.write('<meta http-equiv="Content-Type" content="text/html;charset=UTF-8">')
        self.response.out.write('<script src="http://static.simile.mit.edu/timeline/api-2.3.0/timeline-api.js?bundle=true" type="text/javascript"></script>')
        self.response.out.write('<script src="/js/evernote-timeline.js" type="text/javascript"></script>')
        self.response.out.write('<title>タイムライン表示</title>')
        self.response.out.write('</head><body onload="onLoad()" onresize="onResize()">')
        self.response.out.write('<h1>タイムライン表示</h1>')
        self.response.out.write('<div id="evernote-timeline" class="timeline-default" style="height: 300px; "></div>')
        self.response.out.write('<p><a href="/">トップページに戻る</a></p></body></html>')

class TimelineEventsHandler(webapp.RequestHandler):
    def get(self):
        session = sessions.Session()
        if not check_authenticated_user(session):
            self.redirect("/error")
            return

        notestore = get_notestore(session)
        notebooks = notestore.listNotebooks(session["oauth_token"])
        noteList = notestore.findNotesMetadata(session["oauth_token"], NoteFilter(), 0, 20,
                                               NotesMetadataResultSpec(True, False, True, True, False,
                                                                       False, False, False, False, False))

        timeline_data = { "dateTimeFormat": "iso8601", "events": [] }
        for note in noteList.notes:
            start = time.strftime("%Y-%m-%dT%H:%M:%S%z", time.localtime(note.created / 1000))
            sys.stderr.write(start)
            event = { "start": start, "durationEvent": False, "title": note.title, "description": "" }
            timeline_data["events"].append(event)
            
        self.response.headers["Content-Type"] = "application/json; charset=utf-8"
        self.response.out.write(simplejson.dumps(timeline_data, ensure_ascii=False))
        
class ErrorHandler(webapp.RequestHandler):
    """エラー表示"""
    def get(self):
        self.response.headers["Content-Type"] = "text/html; charset=utf-8"
        self.response.out.write('<html><head><title>エラー</title></head>')
        self.response.out.write('<body><h1>エラー</h1><p><a href="/">トップページに戻る</a></p></body></html>')

class ResetHandler(webapp.RequestHandler):
    """セッションのリセット"""
    def get(self):
        session = sessions.Session()
        session.delete()
        self.response.headers["Content-Type"] = "text/html; charset=utf-8"
        self.response.out.write('<html><head><title>リセット完了</title></head>')
        self.response.out.write('<body><h1>リセット完了</h1><p><a href="/">トップページに戻る</a></p></body></html>')

def main():
    application = webapp.WSGIApplication([('/', IndexHandler),
                                          ('/oauth/token', OAuthTokenHandler),
                                          ('/oauth/token_callback', OAuthTokenCallbackHandler),
                                          ('/list', ListHandler),
                                          ('/timeline', TimelineHandler),
                                          ('/timeline_events', TimelineEventsHandler),
                                          ('/error', ErrorHandler),
                                          ('/reset', ResetHandler)],
                                          debug=True)
    util.run_wsgi_app(application)

if __name__ == '__main__':
    main()
